from typing import Literal

import numpy as np
from pepeline import screentone, cvt_color, CvtType, TypeDot
import cv2 as cv


def rgb_halftone(img: np.ndarray, dot_size: list[int], angle: list[int], dot_type: list[TypeDot]) -> np.ndarray:
    dot_size_len = len(dot_size)
    angle_len = len(angle)
    dot_type_len = len(dot_type)
    if img.ndim == 2:
        img = cvt_color(img, CvtType.GRAY2RGB)
    img[..., 0] = screentone(img[..., 0], dot_size[0 % dot_size_len], angle[0 % angle_len], dot_type[0 % dot_type_len])
    img[..., 1] = screentone(img[..., 1], dot_size[1 % dot_size_len], angle[1 % angle_len], dot_type[1 % dot_type_len])
    img[..., 2] = screentone(img[..., 2], dot_size[2 % dot_size_len], angle[2 % angle_len], dot_type[2 % dot_type_len])
    return img


def cmyk_halftone(img: np.ndarray, dot_size: list[int], angle: list[int], dot_type: list[TypeDot]) -> np.ndarray:
    dot_size_len = len(dot_size)
    angle_len = len(angle)
    dot_type_len = len(dot_type)
    if img.ndim == 2:
        img = cvt_color(img, CvtType.GRAY2RGB)
    img = cvt_color(img, CvtType.RGB2CMYK)
    img[..., 0] = screentone(img[..., 0], dot_size[0 % dot_size_len], angle[0 % angle_len], dot_type[0 % dot_type_len])
    img[..., 1] = screentone(img[..., 1], dot_size[1 % dot_size_len], angle[1 % angle_len], dot_type[1 % dot_type_len])
    img[..., 2] = screentone(img[..., 2], dot_size[2 % dot_size_len], angle[2 % angle_len], dot_type[2 % dot_type_len])
    img[..., 3] = screentone(img[..., 3], dot_size[3 % dot_size_len], angle[3 % angle_len], dot_type[3 % dot_type_len])
    img = cvt_color(img, CvtType.CMYK2RGB)
    return img


def hsv_halftone(img: np.ndarray, dot_size: list[int], angle: list[int], dot_type: list[TypeDot]) -> np.ndarray:
    if img.ndim == 2:
        img = cv.cvtColor(img, cv.COLOR_GRAY2RGB)
    img = cv.cvtColor(img, cv.COLOR_RGB2HSV)
    img[..., 2] = screentone(img[..., 2], dot_size[0], angle[0], dot_type[0])
    img = cv.cvtColor(img, cv.COLOR_HSV2RGB)
    return img


def gray_halftone(img: np.ndarray, dot_size: list[int], angle: list[int], dot_type: list[TypeDot]) -> np.ndarray:
    if img.ndim == 3:
        img = cvt_color(img, CvtType.RGB2GrayBt2020)
    img = screentone(img, dot_size[0], angle[0], dot_type[0])
    return img


MODE_MAP = {'rgb': rgb_halftone, 'gray': gray_halftone, 'hsv': hsv_halftone, 'cmyk': cmyk_halftone}

Mode = Literal['rgb', 'gray', 'hsv', 'cmyk']



from typing import Literal

from chainner_ext.chainner_ext import ResizeFilter

FILTER_MAP = {
    'nearest': ResizeFilter.Nearest,
    'box': ResizeFilter.Box,
    'linear': ResizeFilter.Linear,
    'hermite': ResizeFilter.Hermite,
    'cubic_catrom': ResizeFilter.CubicCatrom,
    'cubic_mitchell': ResizeFilter.CubicMitchell,
    'cubic_bspline': ResizeFilter.CubicBSpline,
    # https://github.com/chaiNNer-org/chaiNNer-rs/issues/28
    #'hamming': ResizeFilter.Hamming,
    #'hann': ResizeFilter.Hann,
    'lanczos': ResizeFilter.Lanczos,
    'lagrange': ResizeFilter.Lagrange,
    'gauss': ResizeFilter.Gauss,
}

FilterType = Literal[
    'nearest',
    'box',
    'linear',
    'hermite',
    'cubic_catrom',
    'cubic_mitchell',
    'cubic_bspline',
    'lanczos',
    'lagrange',
    'gauss',
]



import orjson
import typer
from . import Pipeline

app = typer.Typer(pretty_exceptions_show_locals=False)


@app.command()
def main(config: str = typer.Option('config.json', '--config', '-c', help='Path to the configuration JSON file')):
    try:
        with open(config, encoding='utf-8') as f:
            data = f.read()
            data = orjson.loads(data)

        Pipeline.from_json(data).process_linear()
    except FileNotFoundError:
        typer.echo(f"Error: Config file '{config}' not found.")
        raise typer.Exit(1)
    except orjson.JSONDecodeError:
        typer.echo(f"Error: Invalid JSON in config file '{config}'")
        raise typer.Exit(1)


def cli():
    """Entry point for the CLI."""
    app()


if __name__ == '__main__':
    cli()



from dataclasses import dataclass
from typing import List, Optional, Literal

import numpy as np

from reline.static import Node, NodeOptions, ImageFile
from chainner_ext import quantize, error_diffusion_dither, ordered_dither, riemersma_dither, DiffusionAlgorithm, UniformQuantization

ERROR_DITHERING_MAP = {
    'floydsteinberg': DiffusionAlgorithm.FloydSteinberg,
    'jarvisjudiceninke': DiffusionAlgorithm.JarvisJudiceNinke,
    'stucki': DiffusionAlgorithm.Stucki,
    'atkinson': DiffusionAlgorithm.Atkinson,
    'burkes': DiffusionAlgorithm.Burkes,
    'sierra': DiffusionAlgorithm.Sierra,
    'tworowsierra': DiffusionAlgorithm.TwoRowSierra,
    'sierraLite': DiffusionAlgorithm.SierraLite,
}
DITHERING_TYPE = Literal[
    'floydsteinberg', 'jarvisjudiceninke', 'stucki', 'atkinson', 'burkes', 'sierra', 'tworowsierra', 'sierraLite', 'order', 'riemersma', 'quantize'
]


@dataclass(frozen=True)
class DitheringOptions(NodeOptions):
    colors_per_ch: Optional[int] = 16
    dith_type: Optional[DITHERING_TYPE] = 'sierra'
    map_size: Optional[int] = 8
    history: Optional[int] = 10
    ratio: Optional[float] = 0.5


class DitheringNode(Node[DitheringOptions]):
    def __init__(self, options):
        super().__init__(options)
        self.UQ = UniformQuantization(options.colors_per_ch)

    def _error(self, img: np.ndarray) -> np.ndarray:
        return error_diffusion_dither(img, self.UQ, ERROR_DITHERING_MAP[self.options.dith_type]).squeeze()

    def _order(self, img: np.ndarray) -> np.ndarray:
        return ordered_dither(img, self.UQ, self.options.map_size).squeeze()

    def _riemersma(self, img: np.ndarray) -> np.ndarray:
        return riemersma_dither(img, self.UQ, self.options.history, self.options.ratio).squeeze()

    def _quantize(self, img: np.ndarray) -> np.ndarray:
        return quantize(img, self.UQ).squeeze()

    def process(self, files: List[ImageFile]) -> List[ImageFile]:
        dithering_type_map = {
            'floydsteinberg': lambda img: self._error(img),
            'jarvisjudiceninke': lambda img: self._error(img),
            'stucki': lambda img: self._error(img),
            'atkinson': lambda img: self._error(img),
            'burkes': lambda img: self._error(img),
            'sierra': lambda img: self._error(img),
            'tworowsierra': lambda img: self._error(img),
            'sierraLite': lambda img: self._error(img),
            'order': lambda img: self._order(img),
            'riemersma': lambda img: self._riemersma(img),
            'quantize': lambda img: self._quantize(img),
        }
        dithering_process = dithering_type_map[self.options.dith_type]
        for file in files:
            file.data = dithering_process(img=file.data)

        return files

    def single_process(self, file: ImageFile) -> ImageFile:
        dithering_type_map = {
            'floydsteinberg': lambda img: self._error(img),
            'jarvisjudiceninke': lambda img: self._error(img),
            'stucki': lambda img: self._error(img),
            'atkinson': lambda img: self._error(img),
            'burkes': lambda img: self._error(img),
            'sierra': lambda img: self._error(img),
            'tworowsierra': lambda img: self._error(img),
            'sierraLite': lambda img: self._error(img),
            'order': lambda img: self._order(img),
            'riemersma': lambda img: self._riemersma(img),
            'quantize': lambda img: self._quantize(img),
        }
        dithering_process = dithering_type_map[self.options.dith_type]
        file.data = dithering_process(img=file.data)

        return file

    def video_process(self, file: np.ndarray) -> np.ndarray:
        dithering_type_map = {
            'floydsteinberg': lambda img: self._error(img),
            'jarvisjudiceninke': lambda img: self._error(img),
            'stucki': lambda img: self._error(img),
            'atkinson': lambda img: self._error(img),
            'burkes': lambda img: self._error(img),
            'sierra': lambda img: self._error(img),
            'tworowsierra': lambda img: self._error(img),
            'sierraLite': lambda img: self._error(img),
            'order': lambda img: self._order(img),
            'riemersma': lambda img: self._riemersma(img),
            'quantize': lambda img: self._quantize(img),
        }
        dithering_process = dithering_type_map[self.options.dith_type]
        file = dithering_process(img=file)

        return file
